using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using JetBrains.Annotations;
using Orchard.ContentManagement;
using Orchard.DisplayManagement;
using Orchard.Localization;
using Orchard.Logging;
using Orchard.Settings;
using Contrib.Taxonomies.Services;
using Orchard.Themes;
using Orchard.UI.Navigation;

namespace Contrib.Taxonomies.Controllers {
    [ValidateInput(false), Themed]
    public class HomeController : Controller {
        private readonly ITaxonomyService _taxonomyService;
        private readonly IContentManager _contentManager;
        private readonly ISiteService _siteService;

        public HomeController(
            ITaxonomyService taxonomyService, 
            IContentManager contentManager, 
            IShapeFactory shapeFactory,
            ISiteService siteService)
        {
            _taxonomyService = taxonomyService;
            _contentManager = contentManager;
            _siteService = siteService;
            Shape = shapeFactory;
            T = NullLocalizer.Instance;
        }
        
        dynamic Shape { get; set; }

        protected virtual ISite CurrentSite { get; [UsedImplicitly] private set; }
        
        public ILogger Logger { get; set; }
        public Localizer T { get; set; }

        public ActionResult List(string taxonomySlug, string path, PagerParameters pagerParameters) {
            var pager = new Pager(_siteService.GetSiteSettings(), pagerParameters);

            var taxonomy = _taxonomyService.GetTaxonomyBySlug(taxonomySlug);
            var term = _taxonomyService.GetTermBySlug(taxonomy.Id, path);
            if (term == null) {
                return HttpNotFound();
            }

            // loading categorized content, within page boundaries
            IEnumerable<int> itemIds = _taxonomyService
                .GetAssociatedContentItemsId(term);
            
            var totalItemCount = itemIds.Count();
            
            // loading content items
            var contentItems = itemIds
                .Select(cid => _contentManager.Get(cid, VersionOptions.Published))
                .Where(c => c != null)
                .Skip(pager.GetStartIndex())
                .Take(pager.PageSize);

            var model = Shape.Taxonomy_List();

            var list = Shape.List();
            foreach (var contentItem in contentItems.Where(ic => ic != null)) {
                var shape = _contentManager.BuildDisplay(contentItem, "Summary");
                // Adds the Term to the shape, so that the IShapeTableProvider can detect
                // the current content has a Term, and can generate specific Alternates.
                // Alternates can't be added here or the default ones would prevail
                shape.Term(term);
                shape.Taxonomy(taxonomy);
                // and the standard one if we want to call Display() on it from a specific alternate
                shape.OriginalShape = _contentManager.BuildDisplay(contentItem, "Summary");
                list.Add(shape);
            }

            model.Term(term);
            model.Taxonomy(taxonomy);
            model.List(list);
            model.Pager(Shape.Pager(pager).TotalItemCount(totalItemCount));

            return View((object)model);
        }
    }
}
