using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using Contrib.Reviews.Models;
using Contrib.Voting.Functions;
using Contrib.Voting.Models;
using Gallery.Core.Domain;
using JetBrains.Annotations;
using Microsoft.Http;
using Orchard.ContentManagement;
using Orchard.Data;
using Orchard.Gallery.Interfaces;
using Orchard.Gallery.Models;
using Orchard.Gallery.Services;
using Orchard.Logging;
using Orchard.Services;

namespace Orchard.Gallery.RatingSynchronization {
    [UsedImplicitly]
    public class RatingSynchronizer : ThreadSafeActionBase, IRatingSynchronizer {
        private readonly IRepository<ResultRecord> _resultRecordRepository;
        private readonly IRepository<PackagePartRecord> _packagePartRecordRepository;
        private readonly IRepository<ReviewRecord> _reviewRecordRepository;
        private readonly IPackageService _packageService;
        private readonly IOrchardServices _orchardServices;
        private readonly IClock _clock;
        private readonly INonceCache _nonceCache;

        public ILogger Logger { get; set; }

        private readonly Lazy<string> _serviceRoot;

        public RatingSynchronizer(
            IRepository<ResultRecord> resultRecordRepository,
            IRepository<PackagePartRecord> packagePartRecordRepository,
            IRepository<ReviewRecord> reviewRecordRepository,
            IPackageService packageService,
            IOrchardServices orchardServices,
            IClock clock,
            INonceCache nonceCache) {

            _resultRecordRepository = resultRecordRepository;
            _packagePartRecordRepository = packagePartRecordRepository;
            _reviewRecordRepository = reviewRecordRepository;
            _nonceCache = nonceCache;
            _packageService = packageService;
            _orchardServices = orchardServices;
            _clock = clock;

            Logger = NullLogger.Instance;

            _serviceRoot = new Lazy<string>(() => _orchardServices.WorkContext.CurrentSite.As<GallerySettingsPart>().ServiceRoot);
        }

        protected override void ExecuteThreadSafeAction() {
            DateTime? lastRatingSyncTimeSetting = _orchardServices.WorkContext.CurrentSite.As<GallerySettingsPart>().LastRatingSyncTime;
            DateTime lastRatingSyncTime = lastRatingSyncTimeSetting.HasValue ? lastRatingSyncTimeSetting.Value : new DateTime(1900, 1, 1);

            IEnumerable<ResultRecord> resultRecords = _resultRecordRepository.Fetch(rr => rr.ContentType == "Package" && rr.FunctionName == new Average().Name
                && rr.CreatedUtc > lastRatingSyncTime);
            DateTime utcNow = _clock.UtcNow;
            var packageRatingAggregates = new List<PackageVersionRatings>();
            foreach (var resultRecord in resultRecords) {
                ResultRecord closureResultRecord = resultRecord;
                var package = _packageService.Get(p => p.ContentItemRecord.Id == closureResultRecord.ContentItemRecord.Id).SingleOrDefault();
                if (package != null) {
                    var packageRatingAggregate = new PackageVersionRatings
                    {
                        PackageId = package.PackageID,
                        PackageVersion = package.PackageVersion,
                        RatingAverage = closureResultRecord.Value,
                        RatingCount = closureResultRecord.Count
                    };
                    packageRatingAggregates.Add(packageRatingAggregate);
                }
            }

            if (packageRatingAggregates.Any()) {
                using (var client = new HttpClient(_serviceRoot.Value)) {
                    HttpContent content = HttpContentExtensions.CreateDataContract(packageRatingAggregates);
                    _nonceCache.Nonce = Guid.NewGuid().ToString();
                    string uri = string.Format("{0}/{1}/{2}", ServiceConstants.PackageServiceName, "UpdatePackageRatings", _nonceCache.Nonce);
                    using (HttpResponseMessage response = client.Put(uri, content)) {
                        if (response.StatusCode == HttpStatusCode.OK) {
                            _orchardServices.WorkContext.CurrentSite.As<GallerySettingsPart>().LastRatingSyncTime = utcNow;
                        }
                        else {
                            Logger.Error("Call to UpdatePackageRatings on Gallery.Server failed.");
                        }
                    }
                }
            }

            UpdateReviews();
        }

        private void UpdateReviews() {
            // Update the number of reviews each package has
            var reviews = _reviewRecordRepository.Table.ToList().GroupBy(p => p.ContentItemRecordId);
            foreach (var reviewGroup in reviews) {
                int packageId = reviewGroup.Key;
                int reviewsCount = reviewGroup.Count();

                PackagePartRecord packagePartRecord = _packagePartRecordRepository.Get(p => (p.Id == packageId && (p.Calc_NumberOfReviews == null || p.Calc_NumberOfReviews != reviewsCount)));
                if (packagePartRecord != null) {
                    // If the package with the given id and a different review count was found then we need to update the review count
                    packagePartRecord.Calc_NumberOfReviews = reviewsCount;
                    _packagePartRecordRepository.Update(packagePartRecord);
                }
            }
        }
    }
}